import random
import math
random.seed("personal01/attack-pairs")

class TestGen:
    def __init__(self):
        self.test_number = 0

    def generate_samples(self):
        print('Generating samples...')
        self.print_test(2, 2, 3, [2, 1, 6, 5, 4])
        self.print_test(2, 0, 0, [2, 1, 6, 5, 4])

    def generate_hand_tests(self):
        self.print_test(1, 0, 0, [1, 1])
        self.print_test(1, 0, 2, [1, 1])
        self.print_test(1, 1, 1, [1, 1])
        self.print_test(1, 0, 10**9, [1, 1])
        self.print_test(1, 0, 0, [1, 2])
        self.print_test(1, 0, 1, [1, 2])
        self.print_test(1, 1, 2, [1, 2])
        self.print_test(1, 2, 2, [1, 2])
        self.print_test(1, 2, 2, [2, 1])
        self.print_test(1, 1, 2, [2, 1])
        self.print_test(1, 0, 0, [2, 1])
        self.print_test(1, 1, 1, [2, 1])
        self.print_test(1, 1, 1, [2, 4, 6])
        self.print_test(1, 3, 4, [2, 6, 4])
        self.print_test(2, 3, 3, [2, 6, 4])
        self.print_test(2, 3, 4, [2, 6, 4])
        self.print_test(2, 2, 3, [2, 6, 4])

    def generate_max_tests(self, maxn):
        pass

    def generate_random_test(self, n, k):
        a = []
        for i in range(n):
            a.append(random.randint(1, 10**9))
        diffs = set()
        for i in range(n):
            for j in range(max(0, i - k), i):
                diffs.add(abs(a[i] - a[j]))
        diffs = sorted(list(diffs))
        lrs = []
        if diffs[0] > 0:
            lrs.append([0, diffs[0] - 1])
        for i in range(len(diffs) - 1):
            if diffs[i] != diffs[i + 1] - 1:
                lrs.append([diffs[i] + 1, diffs[i + 1] - 1])
        lrs.append([diffs[-1] + 1, random.randint(diffs[-1] + 1, 10**9)])
        index = random.randint(0, len(lrs) - 1)
        self.print_test(k, lrs[index][0], lrs[index][1], a)

    def fast_generate_random_test(self, n, k):
        a = []
        for i in range(n):
            a.append(random.randint(1, 10**6))
        if random.randint(1, 3) == 1:
            self.print_test(k, 10**6, 10**9, a)
        else:
            index = random.randint(0, n - k - 1)
            a[index + random.randint(1, k)] = a[index] + random.randint(10**6, 10**9)
            self.print_test(k, 10**6, 10**9, a)

    def generate_subtask1(self):
        print('Generating subtask1...')
        self.generate_hand_tests()
        for n in [5, 10, 100, 10**3]:
            for k in [3, int(math.sqrt(n)), n // 2, n - 1]:
                self.generate_random_test(n, k)

    def generate_subtask2(self):
        print('Generating subtask2...')
        for n in [10**4, 10**5]:
            for k in [3, 5, 10, 50, 100]:
                self.fast_generate_random_test(n, k)

    def generate_subtask3(self):
        print('Generating subtask3...')
        for n in [5, 10, 50, 100, 10**3, 10**4, 10**5]:
            a = []
            used = set()
            for i in range(n):
                x = random.randint(1, 10**9)
                while x in used:
                    x = random.randint(1, 10**9)
                a.append(x)
                used.add(x)
            if random.randint(1, 3) == 1:
                self.print_test(random.randint(1, n - 1), 0, 0, a)
            else:
                k = random.randint(1, n - 1)
                index = random.randint(0, n - k - 1)
                a[index + random.randint(1, k)] = a[index]
                self.print_test(k, 0, 0, a)
        n = 10**5
        for k in [10**5 // 3, 10**5 // 2]:
            a = []
            used = set()
            for i in range(n):
                x = random.randint(1, 10**9)
                while x in used:
                    x = random.randint(1, 10**9)
                a.append(x)
                used.add(x)
            self.print_test(k, 0, 0, a)
            index = random.randint(0, n - k - 1)
            a[index + random.randint(1, k)] = a[index]
            self.print_test(k, 0, 0, a)            
            
    def generate_subtask4(self):
        print('Generating subtask4...')
        for n in [5, 10, 50, 100, 10**3, 10**4, 10**5]:
            d = random.randint(2, 100)
            a = []
            used = set()
            for i in range(n):
                x = random.randint(1, (10**9 - d) // d)
                while x in used:
                    x = random.randint(1, (10**9 - d) // d)
                a.append(d * x)
                used.add(x)
            if random.randint(1, 3) == 1:
                self.print_test(random.randint(1, n - 1), 0, d - 1, a)
            else:
                k = random.randint(1, n - 1)
                index = random.randint(0, n - k - 1)
                a[index + random.randint(1, k)] = a[index] + random.randint(0, d - 1)
                self.print_test(k, 0, d - 1, a)
        n = 10**5
        for k in [10**5 // 3, 10**5 // 2]:
            d = random.randint(2, 100)
            a = []
            used = set()
            for i in range(n):
                x = random.randint(1, (10**9 - d) // d)
                while x in used:
                    x = random.randint(1, (10**9 - d) // d)
                a.append(d * x)
                used.add(x)
            self.print_test(k, 0, d - 1, a)
            index = random.randint(0, n - k - 1)
            a[index + random.randint(1, k)] = a[index] + random.randint(0, d - 1)
            self.print_test(k, 0, d - 1, a)

    def generate_subtask5(self):
        print('Generating subtask5...')
        a = [(10**9 - i) for i in range(10**5 - 1)] + [1]
        self.print_test(10**5 - 1, 10**7, 10**9 - 1, a)
        self.print_test(10**5 - 2, 10**7, 10**9 - 1, a)
        self.print_test(10**5 - 1, 10**9, 10**9, a)
        a = [i for i in range(1, 10**5 // 4 + 1)] + [(10**9 - i) for i in range(10**5 // 2)] + [i for i in range(1, 10**5 // 4 + 1)]
        self.print_test(10**5 // 2, 10**5 // 2 - 10, 10**5 // 2 + 15, a)
        a = [i for i in range(1, 10**5 // 4 + 1)] + [i for i in range(1, 10**5 // 4 + 1)] + [(10**9 - i) for i in range(10**5 // 2)]
        self.print_test(10**5 // 2, 10**5 // 2 - 10, 10**5 // 2 + 15, a)
        a = [i for i in range(1, 10**5 + 1)]
        a[10**5 - 8] = 239017
        a[10**5 - 3] = 239027
        self.print_test(5, 8, 10, a)
        a = [i for i in range(1, 10**5 + 1)]
        a[10**5 - 8] = 239017
        a[10**5 - 3] = 339027
        self.print_test(10**5 - 1, 10**5, 10**5 + 20, a)

    def print_test(self, k, l, r, a):
        self.test_number += 1
        print('Generating test #%d' % self.test_number)
        test_name = "{0:0=2d}".format(self.test_number)
        test_file = open(test_name, 'w')
        print(len(a), k, l, r, file=test_file)
        print(' '.join(map(str, a)), file=test_file)
        test_file.close()

    def generate_all_tests(self):
        self.generate_samples()

        self.generate_subtask1()
        self.generate_subtask2()
        self.generate_subtask3()
        self.generate_subtask4()
        self.generate_subtask5()

import time
start = time.time()
writer = TestGen()
writer.generate_all_tests()
finish = time.time()
print('All tests were generated. Elapsed time: ', finish - start)
